==> pandora_box-0.15.0.tar.gz: syd-pandora-0.15.0/src/caps/ambient.rs <==
//! Implementation of Ambient set.

use nix::errno::Errno;

use crate::caps::{errors::CapsError, nr, runtime, Capability, CapsHashSet};

pub fn clear() -> Result<(), CapsError> {
    Errno::result(unsafe { libc::prctl(nr::PR_CAP_AMBIENT, nr::PR_CAP_AMBIENT_CLEAR_ALL, 0, 0, 0) })
        .map(std::mem::drop)
        .map_err(CapsError)
}

pub fn drop(cap: Capability) -> Result<(), CapsError> {
    Errno::result(unsafe {
        libc::prctl(
            nr::PR_CAP_AMBIENT,
            nr::PR_CAP_AMBIENT_LOWER,
            libc::c_uint::from(cap.index()),
            0,
            0,
        )
    })
    .map(std::mem::drop)
    .map_err(CapsError)
}

pub fn has_cap(cap: Capability) -> Result<bool, CapsError> {
    let ret = Errno::result(unsafe {
        libc::prctl(
            nr::PR_CAP_AMBIENT,
            nr::PR_CAP_AMBIENT_IS_SET,
            libc::c_uint::from(cap.index()),
            0,
            0,
        )
    })
    .map_err(CapsError)?;

    match ret {
        0 => Ok(false),
        _ => Ok(true),
    }
}

pub fn raise(cap: Capability) -> Result<(), CapsError> {
    Errno::result(unsafe {
        libc::prctl(
            nr::PR_CAP_AMBIENT,
            nr::PR_CAP_AMBIENT_RAISE,
            libc::c_uint::from(cap.index()),
            0,
            0,
        )
    })
    .map(std::mem::drop)
    .map_err(CapsError)
}

pub fn read() -> Result<CapsHashSet, CapsError> {
    let mut res = super::CapsHashSet::default();
    for c in runtime::thread_all_supported() {
        if has_cap(c)? {
            res.insert(c);
        }
    }
    Ok(res)
}

pub fn set(value: &super::CapsHashSet) -> Result<(), CapsError> {
    for c in runtime::thread_all_supported() {
        if value.contains(&c) {
            raise(c)?;
        } else {
            drop(c)?;
        };
    }
    Ok(())
}

==> pandora_box-0.16.0.tar.gz: syd-pandora-0.16.0/src/caps/ambient.rs <==
//! Implementation of Ambient set.

use nix::errno::Errno;

use crate::caps::{errors::CapsError, nr, runtime, Capabilities, Capability};

pub fn clear() -> Result<(), CapsError> {
    Errno::result(unsafe { libc::prctl(nr::PR_CAP_AMBIENT, nr::PR_CAP_AMBIENT_CLEAR_ALL, 0, 0, 0) })
        .map(std::mem::drop)
        .map_err(CapsError)
}

pub fn drop(cap: Capability) -> Result<(), CapsError> {
    Errno::result(unsafe {
        libc::prctl(
            nr::PR_CAP_AMBIENT,
            nr::PR_CAP_AMBIENT_LOWER,
            libc::c_uint::from(cap.index()),
            0,
            0,
        )
    })
    .map(std::mem::drop)
    .map_err(CapsError)
}

pub fn has_cap(cap: Capability) -> Result<bool, CapsError> {
    let ret = Errno::result(unsafe {
        libc::prctl(
            nr::PR_CAP_AMBIENT,
            nr::PR_CAP_AMBIENT_IS_SET,
            libc::c_uint::from(cap.index()),
            0,
            0,
        )
    })
    .map_err(CapsError)?;

    match ret {
        0 => Ok(false),
        _ => Ok(true),
    }
}

pub fn raise(cap: Capability) -> Result<(), CapsError> {
    Errno::result(unsafe {
        libc::prctl(
            nr::PR_CAP_AMBIENT,
            nr::PR_CAP_AMBIENT_RAISE,
            libc::c_uint::from(cap.index()),
            0,
            0,
        )
    })
    .map(std::mem::drop)
    .map_err(CapsError)
}

pub fn read() -> Result<Capabilities, CapsError> {
    let mut res = Capabilities::empty();

    #[allow(clippy::disallowed_methods)]
    for flag in runtime::thread_all_supported() {
        let cap = flag.try_into().unwrap();

        if has_cap(cap)? {
            res |= flag;
        }
    }

    Ok(res)
}

pub fn set(value: Capabilities) -> Result<(), CapsError> {
    #[allow(clippy::disallowed_methods)]
    for flag in runtime::thread_all_supported() {
        let cap = flag.try_into().unwrap();

        if value.contains(flag) {
            raise(cap)?;
        } else {
            drop(cap)?;
        }
    }

    Ok(())
}
